@@ -34,6 +34,7 @@ import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
 
 import java.io.IOException;
+import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Date;
 
@@ -41,7 +42,6 @@ import java.util.Date;
  * O365 token wrapper
  */
 public class O365Token {
-    protected final String RESOURCE_URL = "https://outlook.office365.com/";
 
     protected static final Logger LOGGER = Logger.getLogger(O365Token.class);
 
@@ -57,14 +57,14 @@ public class O365Token {
     public O365Token(String tenantId, String clientId, String redirectUri, String password) {
         this.clientId = clientId;
         this.redirectUri = redirectUri;
-        this.tokenUrl = "https://login.microsoftonline.com/" + tenantId + "/oauth2/token";
+        this.tokenUrl = Settings.getO365LoginUrl() + tenantId + "/oauth2/token";
         this.password = password;
     }
 
     public O365Token(String tenantId, String clientId, String redirectUri, String code, String password) throws IOException {
         this.clientId = clientId;
         this.redirectUri = redirectUri;
-        this.tokenUrl = "https://login.microsoftonline.com/" + tenantId + "/oauth2/token";
+        this.tokenUrl = Settings.getO365LoginUrl() + tenantId + "/oauth2/token";
         this.password = password;
 
         ArrayList<NameValuePair> parameters = new ArrayList<>();
@@ -164,7 +164,7 @@ public class O365Token {
         parameters.add(new BasicNameValuePair("refresh_token", refreshToken));
         parameters.add(new BasicNameValuePair("redirect_uri", redirectUri));
         parameters.add(new BasicNameValuePair("client_id", clientId));
-        parameters.add(new BasicNameValuePair("resource", "https://outlook.office365.com/"));
+        parameters.add(new BasicNameValuePair("resource", Settings.OUTLOOK_URL));
 
         RestRequest tokenRequest = new RestRequest(tokenUrl, new UrlEncodedFormEntity(parameters, Consts.UTF_8));
 
@@ -177,7 +177,7 @@ public class O365Token {
     private void executeRequest(RestRequest tokenMethod) throws IOException {
         // do not keep login connections open (no pooling)
         try (
-                HttpClientAdapter httpClientAdapter = new HttpClientAdapter(RESOURCE_URL);
+                HttpClientAdapter httpClientAdapter = new HttpClientAdapter(tokenUrl);
                 CloseableHttpResponse response = httpClientAdapter.execute(tokenMethod)
         ) {
             setJsonToken(tokenMethod.handleResponse(response));
@@ -191,7 +191,7 @@ public class O365Token {
     }
 
 
-    static O365Token load(String tenantId, String clientId, String redirectUri, String username, String password) {
+    static O365Token load(String tenantId, String clientId, String redirectUri, String username, String password) throws UnknownHostException {
         O365Token token = null;
         if (Settings.getBooleanProperty("davmail.oauth.persistToken", true)) {
             String encryptedRefreshToken = Settings.loadRefreshToken(username);
@@ -207,8 +207,12 @@ public class O365Token {
                     LOGGER.debug("Authenticated user " + localToken.getUsername() + " from stored token");
                     token = localToken;
 
+                } catch (UnknownHostException e) {
+                    // network down, rethrow to avoid invalidating token
+                    throw e;
                 } catch (IOException e) {
                     LOGGER.error("refresh token failed " + e.getMessage());
+                    // TODO detect network down and rethrow exception
                 }
             }
         }
